<?php

/*

Direct connection:
		  - Test AD/LDAP logon
		  - Get schemas
		  - Trigger backend to update local field list
  - Preview data from ldap/ad/text - for helping user to pick correct fields
		  - Trigger backend to update local data
  - Preview data from local data
		  - Trigger backend to update mapping files

  Later:
  - Validate regular expression for filters
  - 

SQL:
- Get field list
- Send filed list update

*/

/**
 * 
 *
 */
class MappingBackendConnector {
	
	const PERL = ''; ///etc/opt/openit/perl';
	//const PROGRAM = '/opt/openit/bin/map_interface'; // 'home/glarsen/Projects/openit/PSM_Resources/sources/accounting/www/map_interface.pl.sh';

	/*
	const PERL = '/usr/bin/perl';
	const PROGRAM = '/home/glarsen/mapping_connector.pl';
	*/
	
	static private function perl($command, $arg = '') {
		if (strstr(PHP_OS, "WIN")){
			$program = '"' . $_SERVER['DOCUMENT_ROOT'] . "/../../bin/map_interface.bat" . '"';
		} else {
			$program = '/opt/openit/bin/map_interface';
		}
        
		return trim(shell_exec(self::PERL . ' ' . $program . ' ' . $command . ' ' . $arg));        
		//die(self::PERL . ' ' . self::PROGRAM . ' ' . $command . ' ' . $arg);
		
	}
	
	/**
	 * Returns list of availble schemas, for given source id
	 * Only works with LDAP/AD
	 * 
	 * @return mixed List of schemas
	 */
	static public function getSchemas($sourceId) {
		$data = @json_decode(self::perl('ldap-classes', $sourceId));
		if (!is_array($data)) {
			Logger::Error(__CLASS__ . '::' . __FUNCTION__, 'Open iT Backend did not return an schema list/array');
			return array();
		}
	
		return $data;		
	}
	
	
	/**
	 * Testing if backend can connect to AD/LDAP server for the given source
	 * Only works with LDAP/AD
	 * 
	 * @return boolean
	 */
	static public function canConnect($sourceId, $srcvalues) {
		
		// Give the program some time..
		set_time_limit(0);
                
                // 
                $srcarray = array($sourceId);
                foreach ($srcvalues as $srcvalue) {
                    array_push($srcarray, $srcvalue);
                }
                
                $b64vals = base64_encode(implode("+",$srcarray));
		$data = self::perl('validate', '"' . base64_encode($b64vals."GALUT=") . '"');
                
		Logger::Debug('raw', $data);
		
		//$data = strtoupper(@json_decode($d));

		if ($data == '[2]' ) {
			Logger::Error(__CLASS__ . '::' . __FUNCTION__, 'LDAP bind failure');
		}
		
		return $data[1];
	}
	
	
	/**
	 * Get a preview set of data that will come from the data set
	 * 
	 * @return mixed Key value list
	 */
	static public function getPreviewData($sourceId, $recordId) {
		$data = self::perl('source-record', '"' . $sourceId . '"' . ' ' . '"' . $recordId . '"');
/*
		if (!is_array($data)) {
			Logger::Error(__CLASS__ . '::' . __FUNCTION__, 'Open iT Backend did not return preview data');
			return array();
		}
	*/
		return $data;		
	}

	
	/**
	 * Get availble fields for a given source  
	 * 
	 * @return mixed Key value list 
	 */
	static public function getFields($sourceId) {
		$data = @json_decode(self::perl('fields', $sourceId), true);
		
		if (!is_array($data)) {
			Logger::Error(__CLASS__ . '::' . __FUNCTION__, 'Open iT Backend did not return fields from source');
			return array();
		}
	
		return $data;		
	}

	/**
	 * Get a preview set of data that will come from the data set 
	 * 
	 * @return mixed Key value list 
	 */
	public function getPreviewFieldData($sourceId, $recordId) {
		$data = @json_decode($this->perl('fieldpreview', '"' . $sourceId . '"' . '"' . $recordId . '"'), true);
		
		if (!is_array($data)) {
			Logger::Error(__CLASS__ . '::' . __FUNCTION__, 'Open iT Backend did not return preview data from local data set');
			return array();
		}
	
		return $data;		
	}
	
	
	/**
	 * Trigger an event in backend for updateing/create the field list for the given source
	 */
	public function triggerFieldList($sourceId) {
		return;
	}

	
	/**
	 * Trigger an event in backend for updateing/create the data for the given source
	 */
	public function triggerData($sourceId) {
		return;
	}
		
	
	/**
	 * Trigger an event in backend for updateing/create the mapping files for the given source
	 */
	public function triggerMappingFiles($sourceId) {
		return;
	}
	
	
}

?>